window.Gleam.initSearch([{"doc":"lazy_const","title":"lazy_const","content":"# lazy_const\n\n[![Package Version](https://img.shields.io/hexpm/v/lazy_const)](https://hex.pm/packages/lazy_const)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/lazy_const/)\n\nA simple library wrapping [persistent terms](https://www.erlang.org/doc/apps/erts/persistent_term.html)\nand [Maps](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Map)\nto provide lazily initialised constants to Gleam. Please make sure you have read\nthe documentation for your platform implementation and the notes on top of the\n[module docs](lazy_const.html) docs to understand the performance implications.\nAlways measure to make sure using this library makes a difference.\n\n```sh\ngleam add lazy_const@1\n```\n\n```gleam\nimport lazy_const\nimport gleam/regex\n\npub fn main() {\n  // regex is constructed and returned here\n  let re: Regex = alphanum_re()\n\n  // ... do some work ...\n\n  // calling the function again returns the same Regex without recompiling\n  let re: Regex = alphanum_re()\n\n  // ... more very important alphanum-matching ensues ...\n}\n\nfn alphanum_re() -&gt; regex.Regex {\n  use &lt;- lazy_const.new(lazy_const.defined_in(alphanum_re))\n  let assert Ok(re) = regex.from_string(&quot;[a-zA-Z0-9]+&quot;)\n  re\n}\n\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/lazy_const&gt;.\n","url":"index.html"},{"doc":"lazy_const","title":"Id","content":"pub opaque type Id(a)\n A type used to make sure you don&#39;t accidentily pass the constructor and\n wrapper functions the wrong order!\n\nSynonyms:\nlazy_const.Id\nlazy_const Id","url":"lazy_const.html#Id"},{"doc":"lazy_const","title":"defined_in","content":"pub fn defined_in(outer wrapper_function: fn() -&gt; a) -&gt; Id(a)\n Explicitely mark a function as the wrapping function where the constant is\n defined in.\n \n Read the module documentation at the top to learn more about how this\n library works and how to use it!\nSynonyms:\nlazy_const.defined_in\nlazy_const defined_in","url":"lazy_const.html#defined_in"},{"doc":"lazy_const","title":"new","content":"pub fn new(\n  in wrapper_function: Id(a),\n  ctor inner_fn: fn() -&gt; a,\n) -&gt; a\n Define a new lazy constant. The `ctor` will only be called once, after which\n the returned value will be cached, such that future invocations are fast!\n \n **Warning:** Please make sure you have read the docs at the top of the module first.\nSynonyms:\nlazy_const.new\nlazy_const new","url":"lazy_const.html#new"},{"doc":"lazy_const","title":"lazy_const","content":" This is a simple library that allows you to define lazily initialised\n constants, without the usual restrictions around constants in Gleam.\n Examples where this might be useful include only compiling regexes once,\n reading environment variables, or caching and re-using options constructed\n using the builder pattern.\n \n Constants are stored using\n [persistent terms](https://www.erlang.org/doc/apps/erts/persistent_term.html)\n in Erlang, and using a global\n [Map](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Map)\n in Javascript. Make sure you read the documentation for your\n target platform to understand the performance implications.\n \n This library can easily be overused! Only reach for it if you tested and\n measured that it makes a noticable difference! Usually, constructing\n constants is very cheap, even if it looks like duplicate work. Both targets\n already include optimisations to make this faster. In particular, every\n access of a constant defined using `lazy_const` also includes the use of\n a type constructor and a bunch of function calls.\n  \n ### Type Safety\n\n Always create a new top-level wrapper function for each lazy constant.\n The first line of your constant must always look like this, for a constant\n named `my_constant`:\n \n ```gleam\n use &lt;- lazy_const.new(lazy_const.defined_in(my_constant))\n // ...\n ```\n \n The first line must always _use_ `lazy_const.new`, and must call\n `lazy_const.defined_in`, passing the outer wrapping function.\n  \n The outer wrapper function passed as the first argument is used as the key\n inside the cache structure, so a constant is only unique if that key doesn&#39;t\n change, and there can only ever be one constant per `Id` argument.\n \n Functions defined at the top-level are guaranteed to be constant values.\n If a locally defined function is used, a new constant might be created for\n every call, depending on the target and the currently active optimisations,\n degrading performance and using up more and more memory on each call.\n \n\n ### Example usage\n\n ```\n import gleam/regex\n // only ever compile this regex once\n fn alphanum_re() -&gt; regex.Regex {\n   use &lt;- lazy_const.new(lazy_const.defined_in(alphanum_re))\n   let assert Ok(re) = regex.from_string(&quot;[a-zA-Z0-9]+&quot;)\n   re\n }\n \n // an expensive calculation that should only be done once.\n fn fib40() -&gt; Int {\n   use &lt;- lazy_const.new(lazy_const.defined_in(fib40))\n   fib(40)\n }\n \n fn fib(n: Int) -&gt; Int {\n   case n {\n     1 -&gt; 1\n     2 -&gt; 1\n     n -&gt; fib(n - 1) + fib(n - 2)\n   }\n }\n ```\n \n Always prefer few big constants over many small ones.\n\n ### Limitations\n\n You cannot construct cyclical values using this library - calling a \n constant inside of its definition will still lead to a stack overflow.\n \n Constants have to be defined as top-level functions and cannot take any\n arguments.\n \n The target (in particular Erlang) may put additional limits on the amount\n constants defined, and performance may degrade for every additional constant.\n\n While in in most cases, the constructor is only ever called once, this should\n shoul not be relied upon without additional safety nets - the target is in\n theory allowed to evict constants at any time. Every node in a cluster will\n also have its own instance of a constant. ","url":"lazy_const.html"}]);